@@ -5,9 +5,11 @@ import threading
 
 from django.http import HttpResponse, HttpRequest
 
+from . import utils
 from .httpbased import HttpContext, HttpHandler, run_event_loop
+from .remote_access import start_remote_access_service
 from .utils import make_applications, cdn_validation
-from ..utils import STATIC_PATH, iscoroutinefunction, isgeneratorfunction, get_free_port
+from ..utils import STATIC_PATH, iscoroutinefunction, isgeneratorfunction, get_free_port, parse_file_size
 
 logger = logging.getLogger(__name__)
 
@@ -35,12 +37,8 @@ class DjangoHttpContext(HttpContext):
         """返回当前请求的URL参数"""
         return self.request.GET.get(name, default=default)
 
-    def request_json(self):
-        """返回当前请求的json反序列化后的内容，若请求数据不为json格式，返回None"""
-        try:
-            return json.loads(self.request.body.decode('utf8'))
-        except Exception:
-            return None
+    def request_body(self):
+        return self.request.body
 
     def set_header(self, name, value):
         """为当前响应设置header"""
@@ -101,20 +99,15 @@ def webio_view(applications, cdn=True,
 urlpatterns = []
 
 
-def start_server(applications, port=8080, host='', cdn=True, static_dir=None,
+def wsgi_app(applications, cdn=True,
+                 static_dir=None,
                  allowed_origins=None, check_origin=None,
                  session_expire_seconds=None,
                  session_cleanup_interval=None,
-                 debug=False, **django_options):
-    """Start a Django server to provide the PyWebIO application as a web service.
+                 debug=False, max_payload_size='200M', **django_options):
+    """Get the Django WSGI app for running PyWebIO applications.
 
-    :param bool debug: Django debug mode.
-       See `Django doc <https://docs.djangoproject.com/en/3.0/ref/settings/#debug>`_ for more detail.
-    :param django_options: Additional settings to django server.
-       For details, please refer: https://docs.djangoproject.com/en/3.0/ref/settings/ .
-       Among them, ``DEBUG``, ``ALLOWED_HOSTS``, ``ROOT_URLCONF``, ``SECRET_KEY`` are set by PyWebIO and cannot be specified in ``django_options``.
-
-    The rest arguments of ``start_server()`` have the same meaning as for :func:`pywebio.platform.flask.start_server`
+    The arguments of ``wsgi_app()`` have the same meaning as for :func:`pywebio.platform.django.start_server`
     """
     global urlpatterns
 
@@ -124,19 +117,17 @@ def start_server(applications, port=8080, host='', cdn=True, static_dir=None,
     from django.utils.crypto import get_random_string
     from django.views.static import serve
 
-    if port == 0:
-        port = get_free_port()
-
-    if not host:
-        host = '0.0.0.0'
-
     cdn = cdn_validation(cdn, 'warn')
 
+    max_payload_size = parse_file_size(max_payload_size)
+    utils.MAX_PAYLOAD_SIZE = max_payload_size
+
     django_options.update(dict(
         DEBUG=debug,
         ALLOWED_HOSTS=["*"],  # Disable host header validation
         ROOT_URLCONF=__name__,  # Make this module the urlconf
         SECRET_KEY=get_random_string(10),  # We aren't using any security features but Django requires this setting
+        DATA_UPLOAD_MAX_MEMORY_SIZE=max_payload_size
     ))
     django_options.setdefault('LOGGING', {
         'version': 1,
@@ -176,13 +167,46 @@ def start_server(applications, port=8080, host='', cdn=True, static_dir=None,
     if static_dir is not None:
         urlpatterns.insert(0, path(r'static/<path:path>', serve, {'document_root': static_dir}))
 
+    return get_wsgi_application()
+
+
+def start_server(applications, port=8080, host='', cdn=True,
+                 static_dir=None, remote_access=False,
+                 allowed_origins=None, check_origin=None,
+                 session_expire_seconds=None,
+                 session_cleanup_interval=None,
+                 debug=False, max_payload_size='200M', **django_options):
+    """Start a Django server to provide the PyWebIO application as a web service.
+
+    :param bool debug: Django debug mode.
+       See `Django doc <https://docs.djangoproject.com/en/3.0/ref/settings/#debug>`_ for more detail.
+    :param django_options: Additional settings to django server.
+       For details, please refer: https://docs.djangoproject.com/en/3.0/ref/settings/ .
+       Among them, ``DEBUG``, ``ALLOWED_HOSTS``, ``ROOT_URLCONF``, ``SECRET_KEY`` are set by PyWebIO and cannot be specified in ``django_options``.
+
+    The rest arguments of ``start_server()`` have the same meaning as for :func:`pywebio.platform.flask.start_server`
+    """
+    if port == 0:
+        port = get_free_port()
+
+    if not host:
+        host = '0.0.0.0'
+
+    max_payload_size = parse_file_size(max_payload_size)
+    app = wsgi_app(applications, cdn=cdn, static_dir=static_dir, allowed_origins=allowed_origins,
+                   check_origin=check_origin, session_expire_seconds=session_expire_seconds,
+                   session_cleanup_interval=session_cleanup_interval,
+                   debug=debug, max_payload_size=max_payload_size, **django_options)
+
+    if remote_access or remote_access == {}:
+        if remote_access is True: remote_access = {}
+        start_remote_access_service(**remote_access, local_port=port)
+
     use_tornado_wsgi = os.environ.get('PYWEBIO_DJANGO_WITH_TORNADO', True)
     if use_tornado_wsgi:
-        app = get_wsgi_application()  # load app
-
         import tornado.wsgi
         container = tornado.wsgi.WSGIContainer(app)
-        http_server = tornado.httpserver.HTTPServer(container)
+        http_server = tornado.httpserver.HTTPServer(container, max_buffer_size=max_payload_size)
         http_server.listen(port, address=host)
         tornado.ioloop.IOLoop.current().start()
     else:
