@@ -16,7 +16,9 @@ import tornado.ioloop
 from tornado.web import StaticFileHandler
 from tornado.websocket import WebSocketHandler
 
-from .utils import make_applications, render_page, cdn_validation
+from . import utils
+from .remote_access import start_remote_access_service
+from .utils import make_applications, render_page, cdn_validation, deserialize_binary_event
 from ..session import CoroutineBasedSession, ThreadBasedSession, ScriptModeSession, \
     register_session_implement_for_target, Session
 from ..session.base import get_session_info_from_headers
@@ -201,9 +203,13 @@ def _webio_handler(applications=None, cdn=True, reconnect_timeout=0, check_origi
             logger.debug('session id: %s' % self.session_id)
 
         def on_message(self, message):
-            data = json.loads(message)
-            if data is not None:
-                self.session.send_client_event(data)
+            if isinstance(message, bytes):
+                event = deserialize_binary_event(message)
+            else:
+                event = json.loads(message)
+            if event is None:
+                return
+            self.session.send_client_event(event)
 
         def on_close(self):
             cls = type(self)
@@ -246,12 +252,14 @@ async def open_webbrowser_on_server_started(host, port):
     is_open = await wait_host_port(host, port, duration=20)
     if is_open:
         logger.info('Try open %s in web browser' % url)
-        webbrowser.open(url)
+        # webbrowser.open() may block, so invoke it in thread
+        threading.Thread(target=webbrowser.open, args=(url,), daemon=True).start()
     else:
-        logger.error('Open %s failed.' % url)
+        logger.error('Open %s in web browser failed.' % url)
 
 
-def _setup_server(webio_handler, port=0, host='', static_dir=None, **tornado_app_settings):
+def _setup_server(webio_handler, port=0, host='', static_dir=None, max_buffer_size=2 ** 20 * 200,
+                  **tornado_app_settings):
     if port == 0:
         port = get_free_port()
 
@@ -263,18 +271,18 @@ def _setup_server(webio_handler, port=0, host='', static_dir=None, **tornado_app
     handlers.append((r"/(.*)", StaticFileHandler, {"path": STATIC_PATH, 'default_filename': 'index.html'}))
 
     app = tornado.web.Application(handlers=handlers, **tornado_app_settings)
-    server = app.listen(port, address=host)
+    # Credit: https://stackoverflow.com/questions/19074972/content-length-too-long-when-uploading-file-using-tornado
+    server = app.listen(port, address=host, max_buffer_size=max_buffer_size)
     return server, port
 
 
 def start_server(applications, port=0, host='',
                  debug=False, cdn=True, static_dir=None,
+                 remote_access=False,
                  reconnect_timeout=0,
                  allowed_origins=None, check_origin=None,
                  auto_open_webbrowser=False,
-                 websocket_max_message_size=None,
-                 websocket_ping_interval=None,
-                 websocket_ping_timeout=None,
+                 max_payload_size='200M',
                  **tornado_app_settings):
     """Start a Tornado server to provide the PyWebIO application as a web service.
 
@@ -306,6 +314,18 @@ def start_server(applications, port=0, host='',
        The files in this directory can be accessed via ``http://<host>:<port>/static/files``.
        For example, if there is a ``A/B.jpg`` file in ``http_static_dir`` path,
        it can be accessed via ``http://<host>:<port>/static/A/B.jpg``.
+    :param bool/dict remote_access: Whether to enable remote access, when enabled,
+       you can get a temporary public network access address for the current application,
+       others can access your application via this address.
+       Using remote access makes it easy to temporarily share the application with others.
+       The remote access service is provided by `localhost.run <https://localhost.run/>`_.
+       You can use a dict to config remote access service, the following configurations are currently supported:
+
+       - ``ssh_key_path``: Use a custom ssh key, the default key path is ``~/.ssh/id_xxx``. Note that only rsa and ed25519 keys are supported.
+       - ``custom_domain``: Use a custom domain for your remote access address. This need a subscription to localhost.run.
+         See also: `Custom Domains - localhost.run <https://localhost.run/docs/custom-domains/>`_
+
+    :param bool auto_open_webbrowser: Whether or not auto open web browser when server is started (if the operating system allows it) .
     :param int reconnect_timeout: The client can reconnect to server within ``reconnect_timeout`` seconds after an unexpected disconnection.
        If set to 0 (default), once the client disconnects, the server session will be closed.
     :param list allowed_origins: The allowed request source list. (The current server host is always allowed)
@@ -324,41 +344,38 @@ def start_server(applications, port=0, host='',
        It receives the source string (which contains protocol, host, and port parts) as parameter and return ``True/False`` to indicate that the server accepts/rejects the request.
        If ``check_origin`` is set, the ``allowed_origins`` parameter will be ignored.
     :param bool auto_open_webbrowser: Whether or not auto open web browser when server is started (if the operating system allows it) .
-    :param int/str websocket_max_message_size: Max bytes of a message which Tornado can accept.
-        Messages larger than the ``websocket_max_message_size`` (default 10MB) will not be accepted.
-        ``websocket_max_message_size`` can be a integer indicating the number of bytes, or a string ending with `K` / `M` / `G`
+    :param int/str max_payload_size: Max size of a websocket message which Tornado can accept.
+        Messages larger than the ``max_payload_size`` (default 200MB) will not be accepted.
+        ``max_payload_size`` can be a integer indicating the number of bytes, or a string ending with `K` / `M` / `G`
         (representing kilobytes, megabytes, and gigabytes, respectively).
         E.g: ``500``, ``'40K'``, ``'3M'``
-    :param int websocket_ping_interval: If set to a number, all websockets will be pinged every n seconds.
-        This can help keep the connection alive through certain proxy servers which close idle connections,
-        and it can detect if the websocket has failed without being properly closed.
-    :param int websocket_ping_timeout: If the ping interval is set, and the server doesn’t receive a ‘pong’
-        in this many seconds, it will close the websocket. The default is three times the ping interval,
-        with a minimum of 30 seconds. Ignored if ``websocket_ping_interval`` is not set.
     :param tornado_app_settings: Additional keyword arguments passed to the constructor of ``tornado.web.Application``.
         For details, please refer: https://www.tornadoweb.org/en/stable/web.html#tornado.web.Application.settings
     """
-    if websocket_max_message_size:
-        websocket_max_message_size = parse_file_size(websocket_max_message_size)
-    kwargs = locals()
-
     set_ioloop(tornado.ioloop.IOLoop.current())  # to enable bokeh app
 
-    app_options = ['debug', 'websocket_max_message_size', 'websocket_ping_interval', 'websocket_ping_timeout']
-    for opt in app_options:
-        if kwargs[opt] is not None:
-            tornado_app_settings[opt] = kwargs[opt]
-
     cdn = cdn_validation(cdn, 'warn')  # if CDN is not available, warn user and disable CDN
 
+    utils.MAX_PAYLOAD_SIZE = max_payload_size = parse_file_size(max_payload_size)
+
+    tornado_app_settings.setdefault('websocket_max_message_size', max_payload_size)  # Backward compatible
+    tornado_app_settings['websocket_max_message_size'] = parse_file_size(
+        tornado_app_settings['websocket_max_message_size'])
+    tornado_app_settings['debug'] = debug
     handler = webio_handler(applications, cdn, allowed_origins=allowed_origins, check_origin=check_origin,
                             reconnect_timeout=reconnect_timeout)
-    _, port = _setup_server(webio_handler=handler, port=port, host=host, static_dir=static_dir, **tornado_app_settings)
+    _, port = _setup_server(webio_handler=handler, port=port, host=host, static_dir=static_dir,
+                            max_buffer_size=max_payload_size, **tornado_app_settings)
 
     print('Listen on %s:%s' % (host or '0.0.0.0', port))
 
     if auto_open_webbrowser:
         tornado.ioloop.IOLoop.current().spawn_callback(open_webbrowser_on_server_started, host or 'localhost', port)
+
+    if remote_access or remote_access == {}:
+        if remote_access is True: remote_access = {}
+        start_remote_access_service(**remote_access, local_port=port)
+
     tornado.ioloop.IOLoop.current().start()
 
 
@@ -450,7 +467,8 @@ def start_server_in_current_thread_session():
         if os.environ.get("PYWEBIO_SCRIPT_MODE_PORT"):
             port = int(os.environ.get("PYWEBIO_SCRIPT_MODE_PORT"))
 
-        server, port = _setup_server(webio_handler=SingleSessionWSHandler, port=port, host='localhost')
+        server, port = _setup_server(webio_handler=SingleSessionWSHandler, port=port, host='localhost',
+                                     websocket_max_message_size=parse_file_size('4G'))
         tornado.ioloop.IOLoop.current().spawn_callback(partial(wait_to_stop_loop, server=server))
 
         if "PYWEBIO_SCRIPT_MODE_PORT" not in os.environ:
